@@ -18,6 +18,7 @@ import base64
 from typing import List, Tuple, Optional, Union, Type
 from collections import OrderedDict
 import sys
+import zeep
 sys.path.append("..")
 try:
     from custom_exceptions.salesforce_exceptions import *
@@ -634,7 +635,6 @@ class Sf:
             raise ValueError('include_name_compound_fields must be a boolean.')
         return compound_field_names
     
-    
     async def get_picklist_values_by_object_record_type(self, object: str):
         object_record_type_ids = self.query(f"Select Id, Name From RecordType Where SobjectType = '{object}'").apply(lambda row: {"Id": row['Id'], "Name": row['Name']}, axis=1)
         if len(object_record_type_ids) == 0:
@@ -669,9 +669,6 @@ class Sf:
         results = results[['Object', 'RecordTypeName', 'Field_API', 'Controlling Field', 'defaultValueLabel', 'defaultValueValue', 'label', 'value', 'validFor']]
         return results
 
-
-
-
     async def object_permission_bulk_check(self, objects: list, permissions: list ):
         async_object_permission_check = async_wrap(self.object_permission_check)
         result_list = await asyncio.gather(*[async_object_permission_check(object, permission ) for object in objects for permission in permissions])
@@ -691,7 +688,6 @@ class Sf:
         
         return result_dict
     
-    
     async def entity_access_bulk_query(self, entity_types: list):
         async_entity_access_query = async_wrap(self.entity_access_query)
         results = await asyncio.gather(*[async_entity_access_query(entity_type) for entity_type in entity_types])
@@ -701,7 +697,6 @@ class Sf:
                 merged_dict[key] = value
         return merged_dict
 
-
     async def apex_pages_profiles_and_permission_set_access_query(self, apex_pages_to_check: list[str]):
         query_results = await self.entity_access_bulk_query(['ApexPage'])
 
@@ -795,7 +790,60 @@ class Sf:
         else:
             return account_page_layouts_bulk
 
+    async def retrieve_user_profile_record_type_default(self, username: str, objects: list[str]):
+        
+        profile_metadata = pd.DataFrame(zeep.helpers.serialize_object(self.metadata_list("Profile")))
+        profile_metadata = profile_metadata[['id', 'fullName']].copy()
+        profile_metadata
+        user_profile_id = transform_sf_result_set_rec(self.sf.query(f"Select Id, ProfileId FROM User WHERE Username = '{username}'")['records'])
+        user_profile = profile_metadata[profile_metadata['id'] == user_profile_id['User.ProfileId'].values[0]]['fullName'].values[0]
 
+        async_profile_read = async_wrap(self.metadata_read)
+        
+        user_profile_metadata = async_profile_read('Profile', user_profile)
+        
+        results = {}
+        
+        async_object_describe = async_wrap(self.object_describe)
+        
+        tasks = {}
+        
+        for object in objects:
+            tasks[object] = async_object_describe(object)
+        tasks['get_profile_data'] = user_profile_metadata
+        
+        task_result_list = await asyncio.gather(*tasks.values())
+        
+        task_result_dict = {}
+        
+        for object in objects:
+            task_result_dict[object] = task_result_list[objects.index(object)]
+            
+        task_result_dict['get_profile_data'] = task_result_list[-1]
+        
+        user_rt_visibilities = pd.DataFrame(task_result_dict['get_profile_data']['recordTypeVisibilities'][0])
+        user_rt_visibilities['Object'] = user_rt_visibilities['recordType'].apply(lambda x: x.split('.')[0])
+        user_rt_visibilities['RecordType'] = user_rt_visibilities['recordType'].apply(lambda x: x.split('.')[1])
+        
+        for object in objects:
+            if object.lower() == 'account':
+                object_describe = task_result_dict[object]['recordTypeInfos'].T
+                object_record_types = object_describe[object_describe['active'] == True].reset_index()[['name','defaultRecordTypeMapping','developerName','master','recordTypeId']].copy()
+                user_object_rt_visibilities = user_rt_visibilities[(user_rt_visibilities['Object'] == object) | (user_rt_visibilities['Object'] == 'PersonAccount')][['default','personAccountDefault','Object','RecordType','visible']].copy()
+                
+            else:
+                object_describe = task_result_dict[object]['recordTypeInfos'].T
+                object_record_types = object_describe[object_describe['active'] == True].reset_index()[['name','defaultRecordTypeMapping','developerName','master','recordTypeId']].copy()
+                user_object_rt_visibilities = user_rt_visibilities[user_rt_visibilities['Object'] == object][['default','personAccountDefault','Object','RecordType','visible']].copy()
+
+            object_rt_visibility_with_master = pd.merge(user_object_rt_visibilities, object_record_types, left_on='RecordType', right_on='developerName', how='outer')
+            object_rt_visibility_with_master['visible'].fillna(False, inplace=True)
+            if not object_rt_visibility_with_master['visible'].any():
+                object_rt_visibility_with_master.loc[object_rt_visibility_with_master['developerName'] == 'Master', 'visible'] = True
+            visible_object_rt = object_rt_visibility_with_master[object_rt_visibility_with_master['visible'] == True].drop(['defaultRecordTypeMapping'], axis=1)
+            results[object] = visible_object_rt
+        
+        return results
     
     ### ----------------------------------------------------------------------------------------------------
     ### Synchronous Metadata Functions
